/********************************************
*   Orologio micro GT IDE                   *
*   Firmware version: 1.0                   *
*   Autore: Rubens Raffagnato               *
*   Data: 05/02/16                          *
*   Interfaccia diretta Universal Display   *
*   MCU: 16F877A                            *
*   Piattaforma hardware: Micro-GT IDE      *
*   Piattaforma software: MPLAB X v3.15     *
*   Compilatore: Hi-Tech PIC C v. 9.70      *
********************************************/
#define _LEGACY_HEADERS // permette il riconoscimento dei fuses nei nuovi compilatori

//Inclusione file accessori
#include    <pic.h>
#include    "settaggi.h"
#include    "delay.h"
#include    "eeprom.h"
#include    "decoder7seg.h"

//Definizione I/O
#define HDOWN RA3
#define HUP RA2
#define MDOWN RA1
#define MUP RA0
#define dp  RD7

//Definizioni numero display acceso
#define	dsp1	RB0
#define	dsp2	RB1
#define	dsp3	RB2
#define	dsp4	RB3

unsigned int clockMs=0; //Varibile contatore dei millisecondi (dopo 1000ms si azzera e i secondi si incrementano)
unsigned char secondi=0; //Variabile dei secondi
unsigned char minuti=0; //Variabile dei minuti
unsigned char ore=0; //Variabile delle ore
unsigned char clockCalib=0; //Variabile per la calibrazione del clock
unsigned char clockCalibDec=0; //Variabile per la calibrazione dei decimali del clock
unsigned char clockCalibMil=0; //Variabile per la calibrazione dei millesimi del clock
unsigned char clockCalibCount=0; //Varibile necessaria per sommare i decimali alla calibrazione ogni 100s
unsigned char clockCalibMillCount=0; //Varibile necessaria per sommare i millesimi alla calibrazione ogni 10000s
unsigned char calibSign=0; //Variabile che indica il segno positivo o negativo della calibrazione
unsigned char program=0; //Variabile che indica lo stato dell'orologio: 0 = visualizza ora - 1 = in programmazione
unsigned char param=0; //Variabile che indica il parametro attualmente selezionato da settare
unsigned char lampeggio=0; //Variabile che permette il lampeggio dei punti decimali nella fase di programmazione
unsigned char num[4];
unsigned char enFx=0;

__CONFIG (HS & WDTEN & PWRTDIS & BORDIS & LVPDIS & DUNPROT & WRTEN & DEBUGDIS & UNPROTECT);//Fuses
//Predefinizione funzioni
void Settaggi(); //Setta i parametri principali
void VisualizzaOra(); //Invia i comandi al display visualizzando l'ora corrente
void Antirimbalzo(char ms); //Imposta un ritardo per ottenere la funzione antirimbalzo dei pulsanti. Durante il ritardo, esegue un refresh del display per evitarne lo sfarfallio
unsigned char LeggiChar(unsigned char indirizzo); //Legge un byte dalla memoria eeprom del pic

//Routine principale
void main()
{
    Settaggi(); //Richiamo la funzione con i settaggi
    
    dp=1; //Attivo il led di clock
    
    clockCalib=LeggiChar(0); //Leggo le impostazioni di calibrazione dalla memoria eeprom
    clockCalibDec=LeggiChar(1); //Leggo le impostazioni dei decimali della calibrazione dalla memoria eeprom
    clockCalibMil=LeggiChar(2); //Leggo le impostazioni dei millesimi della calibrazione dalla memoria eeprom
    calibSign=LeggiChar(3); //Leggo il segno (+ o -) della calibrazione dalla memoria eeprom

    if(clockCalib==255 || clockCalibDec==255 || clockCalibMil==255 ||calibSign==255) //Verifico se i dati sono stati precedentemente salvati
    {
    //Se equivalgono a 255 (valore di default della memoria) significa che non ho mai settato la calibrazione (prima accensione)

        ScriviChar(0,0); //Quindi inizializzo a 0 la calibrazione nella eeprom
        ScriviChar(0,1); //inizializzo a 0 i decimali della calibrazione nella eeprom
        ScriviChar(0,2); //inizializzo a 0 i millesimi della calibrazione nella eeprom
        ScriviChar(0,3); //e inizializzo a 0 (positivo) il segno della calibrazione nella eeprom

        clockCalib=0; //Inizializzo la calibrazione
        clockCalibDec=0; //inizializzo i decimali della calibrazione
        clockCalibMil=0; //inizializzo i millesimi della calibrazione
        calibSign=0;//e inizializzo il segno a positivo
    }


    while(1){ //Ciclo infinito
        
        if(!HUP || !HDOWN || !MUP || !MDOWN) //Se non sono in programmazione e i pulsanti SET e UP sono premuti
        {
            Antirimbalzo(30); //Pausa per la verifica
            
            if(!HUP){ //Verifico che i pulsanti siano ancora premuti
                if(ore==23){ //Se sono le 23
                    ore=0; //azzera l'ora
                }
                else{ //altrimenti
                    ore++; //incrementa l'ora
                }
            }
            else if(!HDOWN){ //Verifico che i pulsanti siano ancora premuti
                if(ore==0){ //Se sono le 0
                    ore=23; //
                }
                else{ //altrimenti
                    ore--; //decremento l'ora
                }
            }
            else if(!MUP){ //Verifico che i pulsanti siano ancora premuti
                if(minuti==59){ //Se minuti a 59
                    minuti=0; //
                }
                else{ //altrimenti
                    minuti++; //incremento i minuti
                }
            }
            else if(!MDOWN){ //Verifico che i pulsanti siano ancora premuti
                if(minuti==0){ //Se minuti a 0
                    minuti=59; //
                }
                else{ //altrimenti
                    minuti--; //decremento i minuti
                }
            }
        }

        VisualizzaOra(); //Eseguo un ciclo di visualizzazione
    }
}

void interrupt clockTmr0 (void) //Interrupt Timer0
 {
    if (T0IF) //Se l'interrupt  stato generato dal Timer0
    {
        TMR0 = 106; //Ricarico il Timer0 a 61+2 cicli macchina (con quarzo a 20MHz)
        //TMR0 = 133; //Ricarico il Timer0 a 131+2 cicli macchina (con quarzo a 4MHz)
        clockMs++; //Incremento la variabile dei millisecondi
        T0IF=0; //Riabilito l'interrupt sul timer0

        if(program==0) //Stato orologio
        {
            if(clockMs==500) //Se  passato mezzo secondo
            {
                dp=0; //Spengo il led di clock
            }

            if(clockCalibMillCount<100) //Verifica se sono passati 10000s (2h:46m:12s) per sommare o sottrarre (in base al segno) i millesimi della correzione
            {
                if(clockCalibCount<100) //Verifica se sono passati 100s (0h:1m:30s) per sommare o sottrarre (in base al segno) i decimi della correzione
                {
                    if(calibSign==0) //Se il segno della calibrazione  positivo eseguo la somma
                    {
                        if(clockMs==1000+clockCalib) //Se sono passati 1000ms (1s) + il fattore di calibrazione
                        {
                            clockCalibCount++; //Incremento il contatore per sommare i decimali
                            clockMs=0; //Resetto la variabile dei millisecondi
                            secondi++; //Incremento i secondi
                            dp=1; //Riaccendo il led di clock
                        }
                    }
                    else //altrimenti eseguo la sottrazione
                    {
                        if(clockMs==1000-clockCalib) //Se sono passati 1000ms (1s) - il fattore di calibrazione
                        {
                            clockCalibCount++; //Incremento il contatore per sommare i decimali
                            clockMs=0; //Resetto la variabile dei millisecondi
                            secondi++; //Incremento i secondi
                            dp=1; //Riaccendo il led di clock
                        }
                    }
                }
                else
                {
                    if(calibSign==0) //Se il segno della calibrazione  positivo eseguo la somma
                    {
                        if(clockMs==1000+(clockCalib+clockCalibDec)) //Se sono passati 1000ms (1s) + il fattore di calibrazione + il fattore di calibrazione decimale
                        {
                            clockCalibMillCount++; //Incremento il contatore dei millesimi
                            clockCalibCount=0; //Azzero il contatore per sommare i decimali
                            clockMs=0; //Azzero la variabile dei millisecondi
                            secondi++; //Incremento i secondi
                            dp=1; //Riaccendo il led di clock
                        }
                    }
                    else //altrimenti eseguo la sottrazione
                    {
                        if(clockMs==1000-(clockCalib+clockCalibDec)) //Se sono passati 1000ms (1s) - il fattore di calibrazione - il fattore di calibrazione decimale
                        {
                            clockCalibMillCount++; //Incremento il contatore dei millesimi
                            clockCalibCount=0; //Azzero il contatore per sommare i decimali
                            clockMs=0; //Azzero la variabile dei millisecondi
                            secondi++; //Incremento i secondi
                            dp=1; //Riaccendo il led di clock
                        }
                    }
                }
            }
            else
            {
                if(calibSign==0) //Se il segno della calibrazione  positivo eseguo la somma
                {
                    if(clockMs==1000+(clockCalib+clockCalibDec+clockCalibMil)) //Se sono passati 1000ms (1s) + il fattore di calibrazione + il fattore di calibrazione decimale + il fattore di calibrazione dei millesimi
                    {
                        clockCalibMillCount=0; //Azzero il contatore dei millesimi
                        clockCalibCount=0; //Azzero il contatore per sommare i decimali
                        clockMs=0; //Azzero la variabile dei millisecondi
                        secondi++; //Incremento i secondi
                        dp=1; //Riaccendo il led di clock
                    }
                }
                else //altrimenti eseguo la sottrazione
                {
                    if(clockMs==1000-(clockCalib+clockCalibDec+clockCalibMil)) //Se sono passati 1000ms (1s) - il fattore di calibrazione - il fattore di calibrazione decimale - il fattore di calibrazione dei millesimi
                    {
                        clockCalibMillCount=0; //Azzero il contatore dei millesimi
                        clockCalibCount=0; //Azzero il contatore per sommare i decimali
                        clockMs=0; //Azzero la variabile dei millisecondi
                        secondi++; //Incremento i secondi
                        dp=1; //Riaccendo il led di clock
                    }
                }
            }

            if(secondi==60) //Se sono passati 60 secondi
            {
               secondi=0; //Azzero la variabile dei secondi
               minuti++; //Incremento i minuti
               enFx=1;
            }

            if(minuti==60) //Se sono passati 60 minuti
            {
               minuti=0; //Resetto la variabile dei minuti
               ore++; //Incremento le ore
            }

            if(ore==24) //Se sono passate 24 ore
            {
               ore=0; //Resetto la variabile delle ore
            }
        }
        else //Stato programmazione
        {
            if(clockMs==500) //Se  passato mezzo secondo
            {
                lampeggio=0; //Spengo i punti decmali
            }

            if(clockMs==1000) //Se sono passati 1000ms (1s)
            {
                clockMs=0; //Azzero la variabile dei millisecondi
                lampeggio=1; //Riaccendo i punti decimali
            }
        }
    }
}

void VisualizzaOra(){
    char unitaMin;
    char decineMin;
    char unitaOre;
    char decineOre;
    
    dsp1=1;  //Seleziono il display n. 1
    dsp2=0;
    dsp3=0;
    dsp4=0;
    unitaMin=minuti%10; //Prelevo le unit dei minuti
    decoder7seg(unitaMin);
    DelayMs(2);
    PORTB=0;
            
    dsp1=0;  //Seleziono il display n. 2
    dsp2=1;
    dsp3=0;
    dsp4=0;
    decineMin=minuti/10; //Prelevo le decine dei minuti
    decoder7seg(decineMin);
    DelayMs(2);
    PORTB=0;

    dsp1=0;  //Seleziono il display n. 3
    dsp2=0;
    dsp3=1;
    dsp4=0;
    unitaOre=ore%10; //Prelevo le unit delle ore
    decoder7seg(unitaOre);
    DelayMs(2);
    PORTB=0;

    dsp1=0;  //Seleziono il display n. 4
    dsp2=0;
    dsp3=0;
    //dsp4=1;
    if((ore/10)!=0)
    {   
        dsp4=1;
        decineOre=ore/10; //Prelevo le decine delle ore
        decoder7seg(decineOre);
    }
    else
    {
        dsp4=0;
        decineOre=0;
    }
    //decoder7seg(decineOre);
    DelayMs(2);
    PORTB=0;
}

void Antirimbalzo(char ms)
{
    char i;
    for(i=0; i<ms; i++)
    {
        VisualizzaOra(); //Ciclo di visualizzazione
        DelayMs(1); //Pausa di 1 ms
    }
}
